// Based on https://developer.mozilla.org/en/How_to_Build_an_XPCOM_Component_in_Javascript

Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");

function StartupMaster() {
  // constructor
  this.init();
}

StartupMaster.prototype = {
  classDescription: "StartupMaster XPCOM Component",
  classID: Components.ID("{d289d67d-aa42-46bc-b98e-88a708d8a65c}"),
  contractID: "@htamas.ces.hu/startupmaster;1",
  _xpcom_categories: [{category: "profile-after-change", entry: "StartupMaster"}], // Gecko 1.9.2 compatibility
  QueryInterface: XPCOMUtils.generateQI([Components.interfaces.nsIStartupMaster]),

  forcequit: function() {
    try {
      var as = Components.interfaces.nsIAppStartup;
      Components.classes["@mozilla.org/toolkit/app-startup;1"].getService(as).quit(as.eForceQuit);
    } catch(err) {
      var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
      prompts.alert(null, "StartupMaster Error", err);
    }
  },

  pk11login: function(block) {
    try {
      var pk11db = Components.classes["@mozilla.org/security/pk11tokendb;1"].getService(Components.interfaces.nsIPK11TokenDB);
      var token = pk11db.findTokenByName("");
      token.login(false);
    } catch(err) {
      if(block) this.forcequit();
    }
  },

  init: function() {
    var enable = true;
    var block = false;
    try {
      var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
      enable = prefs.getBoolPref("extensions.startupmaster.enable");
      block = prefs.getBoolPref("extensions.startupmaster.block");
    } catch(err) {
    }
    if(enable) {
      this.pk11login(block);
    }
  }
}

if(XPCOMUtils.generateNSGetFactory) {
  // Gecko 2.0 and newer
  var NSGetFactory = XPCOMUtils.generateNSGetFactory([StartupMaster]);
} else {
  // Gecko 1.9.2 compatibility
  var NSGetModule = XPCOMUtils.generateNSGetModule([StartupMaster]);
}

